#include "rfaclient/application.hpp"
#include "rfaclient/manager.hpp"

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/format.hpp>

#include <iostream>
#include <string>

using namespace boost;
using namespace boost::posix_time;
using namespace rfaclient;
using namespace std;

namespace {

    class TestClient : public RfaClientBase {
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       MarketDataMessageType msgType, const Fields& fields)
        {
			cout << "service:" << serviceName << ",item:" << itemName << endl;

            Fields::const_iterator it(fields.begin()), end(fields.end());
            for (; it != end; ++it)
				cout << it->first << "," << it->second << "," << it->second.which() << endl;
              
        }
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       State state, StatusCode statusCode, const string& text)
        {
            
			cout << "service:" << serviceName << ", item:" <<  itemName 
				<< ", state:" << state << ", code:" << statusCode << "[" << text << "]" << endl;
        }
        void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode)
        {
            cout << contId << " " << state << " " << statusCode << endl;
        }
    };
}

int
main(int argc, char* argv[])
{
    try {
		RfaApplication rfa("./ExampleRFA.cfg", 
			"./appendix_a", 
			"./EnumType.def");

	
        TestClient client;
        RfaManager manager("COMETSession", client, 1000, 0);
        manager.subscribe("IDN_SELECTFEED", "EUR=EBS");

		::Sleep(10000);
		cout << "Timeout" << endl;

		manager.unsubscribe("IDN_SELECTFEED", "EUR=");

        /*manager.subscribe("IDN_SELECTFEED", "CHF=");
        manager.subscribe("IDN_SELECTFEED", "JPY=");
        manager.subscribe("IDN_SELECTFEED", "GBP=");
        manager.subscribe("IDN_SELECTFEED", "AUD=");

        manager.subscribe("IDN_SELECTFEED", "CHF=EBS");
        manager.subscribe("IDN_SELECTFEED", "EUR=EBS");
        manager.subscribe("IDN_SELECTFEED", "EURCHF=EBS");
        manager.subscribe("IDN_SELECTFEED", "EURJPY=EBS");
        manager.subscribe("IDN_SELECTFEED", "GBP=D2");
        manager.subscribe("IDN_SELECTFEED", "EURGBP=D2");
        manager.subscribe("IDN_SELECTFEED", "AUD=D2");
        manager.subscribe("IDN_SELECTFEED", "CAD=D2");
        manager.subscribe("IDN_SELECTFEED", "NZD=D2");
        manager.subscribe("IDN_SELECTFEED", "EURSEK=D2");
        manager.subscribe("IDN_SELECTFEED", "EURNOK=D2");*/

       /* manager.clear();

        static const char CCYCCYS[][7] = {
            "EURUSD",
            "USDCHF",
            "EURCHF",
            "USDJPY",
            "EURJPY",
            "GBPUSD",
            "EURGBP",
            "AUDUSD",
            "USDCAD",
            "NZDUSD",
            "EURNOK",
            "EURSEK",
            "XXXYYY"
        };

        
            for (unsigned j(0); j < sizeof(CCYCCYS) / sizeof(CCYCCYS[0]);
                 ++j) {
					 cout << "Subscribing to " << CCYCCYS[j] << endl;
					 manager.subscribe("FXPROP_DTS", str(format("CCYCCY(%1%)")
                                                    % CCYCCYS[j]));
            }

		    ::Sleep(100000);

            for (unsigned j(0); j < sizeof(CCYCCYS) / sizeof(CCYCCYS[0]);
				++j) {
					 cout << "Unsubscribing from " << CCYCCYS[j] << endl;                
					 manager.unsubscribe("FXPROP_DTS", str(format("CCYCCY(%1%)")
                                                      % CCYCCYS[j]));
            }
			::Sleep(1);*/
        
        manager.clear();
        return 0;

    } catch (const rfa::common::Exception& e) {
		cerr << e.getClassification() << e.getErrorType() << e.getServerity() << endl;
    } catch (const exception& e) {
		cerr << e.what() << endl;
    }
    return 1;
}
