#ifndef COMETRFAPP_BASICPUBLISHER_HPP
#define COMETRFAPP_BASICPUBLISHER_HPP

#include "manager.hpp"

#include <set>

namespace rfaclient {

    typedef boost::shared_ptr<Fields> FieldsPtr;

    class  RfaBasicPublisher : private RfaClientBase {

        //! The boolean flag indicates whether an initial snapshot has been
        //! received.

        //! \todo if a connection to DTS is lost, reset the image flag.

        typedef std::map<std::string,
                         std::pair<std::queue<FieldsPtr>, bool> > Items;

        const std::string serviceName_;
        const unsigned timeout_;
        LoggerPtr logger_;
        mutable boost::mutex mutex_;
        boost::condition cond_;
        Items items_;
        unsigned pending_;
        std::set<long> contIds_;
        boost::scoped_ptr<RfaManager> manager_;
        bool publish_;

        void
        publish(const std::string& itemName, const FieldsPtr& fields,
                bool image);

        void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName,
                       MarketDataMessageType msgType, const Fields& fields);

        void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName, State state,
                       StatusCode statusCode, const std::string& text);

        void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode);

    public:
        ~RfaBasicPublisher() NOTHROW;

        RfaBasicPublisher(const std::string& serviceName, unsigned timeout,
                          const LoggerPtr& logger = LoggerPtr());

        //! Without a subscription, RFA publishing will not succeed.  This
        //! component deals with this peculiarity behind the scenes.  The
        //! prepare() function allows users to specify, in advance, which
        //! items they will be publishing to.

        void
        prepare(const std::string& itemName);

        void
        publish(const std::string& itemName, const Fields& fields)
        {
            publish(itemName, FieldsPtr(new Fields(fields)));
        }

        void
        publish(const std::string& itemName, const FieldsPtr& fields);

        bool
        empty() const;
    };
}

#endif // COMETRFAPP_BASICPUBLISHER_HPP
