#define rfaclientRFAPP_BUILD
#include "client.hpp"

#include <SessionLayer/MarketDataItemSub.h>
#include <iostream>

using namespace rfaclient;
using namespace std;


RfaClientBase::~RfaClientBase() 
{
}

 void
RfaClientAdapter::processEvent(const string& serviceName,
                               const string& itemName,
                               MarketDataMessageType msgType,
                               const rfa::common::Buffer& buffer)
{
    TibMsg tMsg;

    TibErr err(tMsg.UnPack((char*)buffer.c_buf(), buffer.size()));
	if (err.code) {
		//cerr << err.code << endl;
		//cerr << err.type << endl;
		//cout <<  string(TibErr::TibMsgText(TibMsg_error(err.code))) << endl;
		throw MarketDataError(string("TibMsg::UnPack() failed: " ) 
			+ string(TibErr::TibMsgText(TibMsg_error(err.code))));
	}


    Fields fields;
    TibField field;
    for (field.First(&tMsg); field.status == TIBMSG_OK;
         field.Next()) {
        try {
            Field local;
			if (unpack(local, field)) {
                fields.insert(make_pair(field.Name(), local));
			}
        } catch (const MarketDataError& e) {
			cerr << e.what() << endl;
            // Continue to unpack other fields - the client may not be
            // interested in the one that failed.
        }
    }


	
	if (!fields.empty()) {
        client_.processEvent(serviceName, itemName, msgType, fields);
	}
}

 void
RfaClientAdapter::processEvent(const rfa::sessionLayer
                               ::MarketDataItemEvent& event)
{
    string serviceName(event.getServiceName().length()
                                ? event.getServiceName() : static_cast<
                                const rfa::sessionLayer::MarketDataItemSub&>
                                (event.getInterestSpec()).getServiceName());

    string itemName(event.getItemName().length()
                             ? event.getItemName() : static_cast<
                             const rfa::sessionLayer::MarketDataItemSub&>
                             (event.getInterestSpec()).getItemName());

    const rfa::common::Buffer& buffer = event.getBuffer();

    // Process the data if it is available.
    if (!buffer.isEmpty()) {
        MarketDataMessageType msgType(event.getMarketDataMsgType());
        processEvent(serviceName, itemName, msgType, buffer);

    } else {
		
        const rfa::sessionLayer
            ::MarketDataItemStatus& status(event.getStatus());
        client_.processEvent(serviceName, itemName, status.getState(),
                             status.getStatusCode(),
                             status.getStatusText());
    }

}

 void
RfaClientAdapter::processEvent(const rfa::sessionLayer
                               ::MarketDataItemContReplyEvent& event)
{
    rfa::sessionLayer::MarketDataItemContReplyStatus status
        (event.getStatus());
    client_.processEvent(event.getContID(), status.getState(),
                         static_cast<ContReplyStatusCode>
                         (status.getStatusCode()));
}


RfaClientAdapter::~RfaClientAdapter() 
{
}


RfaClientAdapter::RfaClientAdapter(RfaClientBase& client)
    : client_(client)
{
}

 void
RfaClientAdapter::processEvent(const rfa::common::Event& event)
{
	// NOTE: we can clone() the event if we wish to retain a copy of it 
	// Must call the destroy() method when we are done
    try {
        switch (event.getType()) {
        case rfa::sessionLayer::MarketDataItemEventEnum:
            processEvent(static_cast<const rfa::sessionLayer
                         ::MarketDataItemEvent&>(event));
            break;
        case rfa::sessionLayer::MarketDataSvcEventEnum:
            break;
        case rfa::sessionLayer::ConnectionEventEnum:
            break;
        case rfa::sessionLayer::MarketDataItemContReplyEventEnum:
            processEvent(static_cast<const rfa::sessionLayer
                         ::MarketDataItemContReplyEvent&>(event));
            break;
        default:
            break;
        }
        if (event.isEventStreamClosed())
          cerr << "Event stream closed" << endl;
    } catch (const rfa::common::Exception& e) {
		cerr << e.getClassification() << endl;
    } catch (const exception& e) {
		cerr << e.what() << endl;
    }
}
