#ifndef rfaclientRFAPP_CLIENT_HPP
#define rfaclientRFAPP_CLIENT_HPP

#include "utility.hpp"

#include <SessionLayer/MarketDataItemContReplyEvent.h>

#include <map>
/**
* Client handler implementation
*/
namespace rfaclient {

	class  RfaClientBase {
        virtual void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName,
                       MarketDataMessageType msgType,
                       const Fields& fields) = 0;

        virtual void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName, State state,
                       StatusCode statusCode, const std::string& text) = 0;

        virtual void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode) = 0;

    public:
        virtual 
        ~RfaClientBase();

        void
        processEvent(const std::string& serviceName,
                     const std::string& itemName,
                     MarketDataMessageType msgType, const Fields& fields)
        {
            doProcessEvent(serviceName, itemName, msgType, fields);
        }

        void
        processEvent(const std::string& serviceName,
                     const std::string& itemName, State state,
                     StatusCode statusCode, const std::string& text)
        {
            doProcessEvent(serviceName, itemName, state, statusCode, text);
        }

        void
        processEvent(long contId, ContReplyState state,
                     ContReplyStatusCode statusCode)
        {
            doProcessEvent(contId, state, statusCode);
        }
    };

    class  RfaClientAdapter : public rfa::common::Client {

        RfaClientBase& client_;
      
        void
        processEvent(const std::string& serviceName,
                     const std::string& itemName,
                     MarketDataMessageType msgType,
                     const rfa::common::Buffer& buffer);

        void
        processEvent(const rfa::sessionLayer::MarketDataItemEvent& event);

        void
        processEvent(const rfa::sessionLayer
                     ::MarketDataItemContReplyEvent& event);

    public:
        virtual
        ~RfaClientAdapter();

		explicit
        RfaClientAdapter(RfaClientBase& client);

        // Client virtual functions.

        void
        processEvent(const rfa::common::Event& event);
    };
}

#endif // rfaclientRFAPP_CLIENT_HPP
