#define rfaclientRFAPP_BUILD

#include <iostream>

#include "dispatcher.hpp"
#include "system.hpp"

#include <R.h>

using namespace boost;
using namespace rfaclient;
using namespace std;


RfaDispatcher::~RfaDispatcher()
{
    try {
        stop();
    } catch (const rfa::common::Exception& e) {
		cerr << e.getErrorType() << endl;
    } catch (const exception& e) {
		cerr << e.what() << endl;
    }
}


RfaDispatcher::RfaDispatcher(unsigned timeout)
    : timeout_(timeout),
      stop_(false)
{
    thread_.reset(new thread(ref(*this)));
}


RfaDispatcher::RfaDispatcher(unsigned timeout,
                             const function0<void>& timeoutFn)
    : timeout_(timeout),
      timeoutFn_(timeoutFn),
      stop_(false)
{
    thread_.reset(new thread(ref(*this)));
}

 void
RfaDispatcher::stop()
{
    mutex::scoped_lock lock(mutex_);
    if (stop_)
        return;
    stop_ = true;
    lock.unlock();
    thread_->join();
}

 void
RfaDispatcher::addEventQueue(const EventQueuePtr& eventQueue)
{
    mutex::scoped_lock lock(mutex_);
    pending_.push(eventQueue);
}

 void
RfaDispatcher::operator ()()
{
    vector<EventQueuePtr> eventQueues; // Should outlive group.
    EventQueueGroupPtr eventQueueGroup;

    try {

        eventQueueGroup = createEventQueueGroup(RFA_String());
        mutex::scoped_lock lock(mutex_);

        do {

            while (!pending_.empty()) {
                EventQueuePtr eventQueue(pending_.front());
                eventQueues.push_back(eventQueue);
                pending_.pop();
                eventQueueGroup->addEventQueue(*eventQueue);
            }
            lock.unlock();

            if (!eventQueueGroup->hasActiveEventStreams()) {
                msleep(timeout_);
                lock.lock();
                continue; // Again.
            }

            try {

                // Catch and log errors in dispatch functions
                bool timeout(true);
                switch (eventQueueGroup->dispatch(timeout_)) {
                case Dispatchable::NothingDispatched:
                    break;
                case Dispatchable::NothingDispatchedInActive:
                    break;
                case Dispatchable::NothingDispatchedPartOfGroup:
                    //cerr << "Event queue is part of a group and should not be used directly!" << endl;
                    break;
                case Dispatchable::NothingDispatchedNoActiveEventStreams:
                    break;
                default:
                    timeout = false;
                }

                if (timeout && !timeoutFn_.empty())
                    timeoutFn_();

            } catch (const rfa::common::Exception& e) {
				cerr << e.getClassification() << endl;
            } catch (const exception& e) {
				cerr << e.what() << endl;
            }

            // Lock before testing stop_ flag.
            lock.lock();

        } while (!stop_);

    } catch (const rfa::common::Exception& e) {
		cerr << e.getErrorType() << endl;
    } catch (const exception& e) {
		cerr << e.what() << endl;
    }

    if (eventQueueGroup) {

        vector<EventQueuePtr>::const_iterator it(eventQueues.begin()),
            end(eventQueues.end());
        for (; it != end; ++it)
            eventQueueGroup->removeEventQueue(**it);
    }
}
