#define rfaclientRFAPP_BUILD

#include <iostream>

#include "logger.hpp"

#include <Logger/LoggerEnums.h>
#include <Logger/LoggerNotifyEvent.h>

#include <boost/format.hpp>

using namespace boost;
using namespace rfaclient;
using namespace rfa;
using namespace std;

using namespace config;
using namespace logger;


/**
* Basic client implementation that just logs 
* any received events
*/

RfaLoggerClient::~RfaLoggerClient() 
{
}

 void
RfaLoggerClient::processEvent(const Event& event)
{
    if (event.getType() == LoggerNotifyEventEnum) {

        try {

            const LoggerNotifyEvent& le
                (static_cast<const LoggerNotifyEvent&>(event));

            cerr << str
                     (format("RFA: component=%1%, severity=%2%,"
                             " logid=%3%, text=%4%")
                      % le.getComponentName()
                      % le.getSeverity()
                      % le.getLogID()
                      % le.getMessageText()) << endl;

        } catch (const rfa::common::Exception& e) {
            cerr << toString(e) << endl;
        } catch(const exception& e) {
            cerr << e.what() << endl;
        }
    }
}


RfaLogger::~RfaLogger() 
{
    try {
        if (handle_)
            appLoggerMonitor_->unregisterLoggerClient(handle_);
    } catch (const rfa::common::Exception& e) {
        cerr << toString(e) << endl;
    } catch (const exception& e) {
        cerr << e.what() << endl;
    }
}


RfaLogger::RfaLogger()
try : applicationLogger_(createApplicationLogger()),
      appLoggerMonitor_(createAppLoggerMonitor(*applicationLogger_,
                                               RFA_String())),
      eventQueue_(createEventQueue(RFA_String())),
      client_(),
      handle_(0)
{
    AppLoggerInterestSpec appLoggerInterestSpec;
    appLoggerInterestSpec.setMinSeverity(rfa::common::Success);

    handle_ = appLoggerMonitor_
        ->registerLoggerClient(*eventQueue_, appLoggerInterestSpec,
                               client_, 0);
} catch (const rfa::common::Exception& e) {
    throw MarketDataError(toString(e));
}
