#define rfaclientRFAPP_BUILD
#define rfaclientRFAPP_BUILD

#include <iostream>

#include "manager.hpp"

using namespace boost;
using namespace rfaclient;
using namespace std;


RfaManager::~RfaManager() 
{
    try {
        dispatcher_.stop();
    } catch (const rfa::common::Exception& e) {
		cerr << toString(e) << endl;
    } catch (const exception& e) {
		cerr << e.what() << endl;
    }
}


RfaManager::RfaManager(const string& sessionName, RfaClientBase& client,
                       unsigned timeout)
try : session_(createSession(RFA_String(sessionName))),
      eventQueue_(createEventQueue(RFA_String())),
      clientAdapter_(client),
      subscriber_(session_, eventQueue_, clientAdapter_),
      dispatcher_(timeout)
{
    dispatcher_.addEventQueue(eventQueue_);
} catch (const rfa::common::Exception& e) {
    throw MarketDataError(toString(e));
}


RfaManager::RfaManager(const string& sessionName, RfaClientBase& client,
                       unsigned timeout, const function0<void>& timeoutFn)
try : session_(createSession(RFA_String(sessionName))),
      eventQueue_(createEventQueue(RFA_String())),
      clientAdapter_(client),
      subscriber_(session_, eventQueue_, clientAdapter_),
      dispatcher_(timeout, timeoutFn)
{
    dispatcher_.addEventQueue(eventQueue_);
} catch (const rfa::common::Exception& e) {
    throw MarketDataError(toString(e));
}


RfaManager::RfaManager(const string& sessionName, RfaClientBase& client,
					   const PrincipalIdentity* identity,
                       unsigned timeout, const function0<void>& timeoutFn)
try : session_(createSession(RFA_String(sessionName), identity)),
      eventQueue_(createEventQueue(RFA_String())),
      clientAdapter_(client),
      subscriber_(session_, eventQueue_, clientAdapter_),
      dispatcher_(timeout, timeoutFn)
{
    dispatcher_.addEventQueue(eventQueue_);
} catch (const rfa::common::Exception& e) {
    throw MarketDataError(toString(e));
}



 void
RfaManager::subscribe(const string& serviceName, const string& itemName)
{
	const rfa::common::RFA_Vector<rfa::common::RFA_String> dummyFids;
    RfaSubscriptionPtr ptr(subscriber_.createSubscription(serviceName,
                                                          itemName));

    subscriptions_
        .insert(make_pair(make_pair(serviceName, itemName), ptr));

}



 void
RfaManager::unsubscribe(const string& serviceName, const string& itemName)
{
    subscriptions_.erase(make_pair(serviceName, itemName));
}

 void
RfaManager::clear()
{
    subscriptions_.clear();
}
