#ifndef COMETRFAPP_POSITIONPUBLISHER_HPP
#define COMETRFAPP_POSITIONPUBLISHER_HPP

#include "client.hpp"
#include "manager.hpp"

#include "cometbasepp/datastore.hpp" // Position

namespace rfaclient {

    struct RfaPositionSettings {
        std::string sessionName_;
        unsigned dispatchTimeout_;
        std::string serviceName_;
        std::string positionName_;
        std::string plRateName_;
        std::string symbolsField_;
        std::string timestampField_;
        std::string statusField_;
        std::string fieldTemplate_;
    };

    class RfaPositionPublisher : private RfaClientBase {

        boost::mutex mutex_;
        const std::vector<std::pair<std::string, double> > initial_;
        const RfaPositionSettings settings_;
        LoggerPtr logger_;
        Fields positions_, plRates_;
        boost::scoped_ptr<RfaManager> manager_;
        bool open_, publish_;

        void
        publish();

        void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName,
                       MarketDataMessageType msgType, const Fields& fields);

        void
        doProcessEvent(const std::string& serviceName,
                       const std::string& itemName,
                       State state, StatusCode statusCode,
                       const std::string& text);

        void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode);

    public:
        ~RfaPositionPublisher() NOTHROW;

        RfaPositionPublisher(const std::vector<
                             std::pair<std::string, double> >& initial,
                             const RfaPositionSettings& positionSettings,
                             const LoggerPtr& logger = LoggerPtr());

        void
        putPosition(const std::string& currency, double value);

        void
        putPositions(const std::vector<
                     std::pair<std::string, double> >& positions);

        void
        putPlRate(const std::string& currency, double value);

        void
        putPlRates(const std::vector<
                   std::pair<std::string, double> >& plRates);

        void
        republish();

        void
        setOpen(bool open = true);
    };
}

#endif // COMETRFAPP_POSITIONPUBLISHER_HPP
