#define COMETRFAPP_BUILD
#include "publisher.hpp"

#include <SessionLayer/MarketDataItemCont.h>
#include <SessionLayer/MarketDataContributorInterestSpec.h>

#include <iomanip>
#include <sstream>

using namespace boost;
using namespace rfaclient;
using namespace std;

namespace rfaclient {

    const char FS('\034');
    const char GS('\035');
    const char RS('\036');
    const char US('\037');
    const char RTL('1');

    void
    addHeader(ostream& os, const string& itemName)
    {
        os << FS
           << "316" // "Update" type (as opposed to "Verify")
           << US
           << "ZZ"
           << GS
           << itemName
           << US
           << 1; // the RTL
    }

    void
    addValue(ostream& os, const string& name, const Field& value)
    {
        TibMsg_mfeed* mfeed(TibMsg::Fname2Mfeed(name.c_str()));
        if (!mfeed)
            throw MarketDataError(string("TibMsg::Fname2Mfeed() failed: ")
                                  .append(name));

        Tib_i32 fid(TibMsg::Mfeed2Fid(mfeed));
        os << RS
           << fid
           << US
           << setprecision(numeric_limits<double>::digits10)
           << setiosflags(ios::fixed)
           << value;
    }

    void
    addFooter(ostream& str)
    {
        // Add a final FS to finish the message.

        str << FS;
    }
}


RfaPublisher::~RfaPublisher() 
{
    try {
        if (handle_)
            contributor_->unregisterClient(*handle_);
    } catch (const rfa::common::Exception& e) {
        writeLog(LOGERROR, SUBMARKETDATA, COMET_HERE(toString(e)), logger_);
    } catch(const exception& e) {
        writeLog(LOGERROR, SUBMARKETDATA, COMET_HERE(e.what()), logger_);
    }
}


RfaPublisher::RfaPublisher(const SessionPtr& session,
                           const EventQueuePtr& eventQueue,
                           rfa::common::Client& client,
                           const LoggerPtr& logger)
    : session_(session),
      eventQueue_(eventQueue),
      client_(client),
      logger_(logger)
{
    try {
        contributor_ = createMarketDataContributor(*session, RFA_String());

        rfa::sessionLayer::MarketDataContributorInterestSpec interest;
        interest.setContReplyInterest(true);
        interest.setConnectionInterest(true);
        interest.setEntitlementsInterest(true);

        handle_ = contributor_->registerClient(*eventQueue, interest, client,
                                               0);
        if (!handle_)
            throw MarketDataError
                ("MarketDataContributorr::registerClient() failed");

    } catch (const rfa::common::Exception& e) {
        throw MarketDataError(toString(e));
    }
}

 long
RfaPublisher::publish(const string& serviceName, const string& itemName,
                      const Fields& fields)
{
    long id;
    try {

        // Create the wierd RFA message format...

        ostringstream os;
        addHeader(os, itemName);
        Fields::const_iterator it(fields.begin()), end(fields.end());
        for (; it != end; ++it)
            addValue(os, it->first, it->second);
        addFooter(os);

        rfa::sessionLayer::MarketDataItemCont marketDataItemCont;
        marketDataItemCont.setServiceName(RFA_String(serviceName));
        marketDataItemCont.setSubject(RFA_String());
        marketDataItemCont.setItemName(RFA_String(itemName));

        // The buffer takes ownership of the memory passed to it so make a
        // copy.  Also it expects unsigned char and c_str() returns signed
        // char.

        string s(os.str());
        Buffer buffer(reinterpret_cast<const unsigned char*>(s.data()),
                      s.size());
        marketDataItemCont.setBuffer(buffer);

        // Contribute data.

        id = contributor_->contribute(marketDataItemCont);

    } catch (const rfa::common::Exception& e) {
        throw MarketDataError(toString(e));
    }
    return id;
}
