#define rfaclientRFAPP_BUILD
#include "subscriber.hpp"

#include <iostream>

#include <SessionLayer/MarketDataItemSub.h>
#include <SessionLayer/MarketDataSubscriberInterestSpec.h>

using namespace boost;
using namespace rfaclient;
using namespace std;

/**
* Unsubscribe a single subscribed handle
* Can also use unsubscribeAll()
*/

RfaSubscription::~RfaSubscription() 
{
    try {
        if (handle_)
            subscriber_.unsubscribe(*handle_);
    } catch (const rfa::common::Exception& e) {
		cerr << e.getClassification() << endl;
    } catch(const exception& e) {
		cerr << e.what() << endl;
    }
}


RfaSubscription::RfaSubscription(rfa::sessionLayer
                                 ::MarketDataSubscriber& marketDataSubscriber,
                                 Handle* handle)
    : subscriber_(marketDataSubscriber),
      handle_(handle)
{
	
}

 RfaSubscriptionPtr
rfaclient::createSubscription(rfa::sessionLayer
                          ::MarketDataSubscriber& marketDataSubscriber,
                          rfa::common::EventQueue& eventQueue,
                          const string& serviceName, const string& itemName,
                          rfa::common::Client& client)
{
    RfaSubscriptionPtr subscription;
    try {

        rfa::sessionLayer::MarketDataItemSub marketDataItemSub;
        marketDataItemSub.setServiceName(RFA_String(serviceName));
        marketDataItemSub.setItemName(RFA_String(itemName));
        marketDataItemSub.setSubject(RFA_String());

		// NOTE: The last parameter is an opaque handle (closure) that can be used
		// to maintain state
	
        Handle* handle(marketDataSubscriber
                       .subscribe(eventQueue, marketDataItemSub, client, 0));
        if (!handle)
            throw MarketDataError("MarketDataSubscriber::subscribe() failed");

        subscription = RfaSubscriptionPtr
            (new RfaSubscription(marketDataSubscriber, handle));


    } catch (const rfa::common::Exception& e) {
        throw MarketDataError(toString(e));
    }
    return subscription;
}


RfaSubscriber::~RfaSubscriber() 
{
    try {
        if (handle_)
            subscriber_->unregisterClient(*handle_);
    } catch (const rfa::common::Exception& e) {
		cerr << e.getClassification() << endl;
    } catch(const exception& e) {
		cerr << e.what() << endl;
    }
}


RfaSubscriber::RfaSubscriber(const SessionPtr& session,
                             const EventQueuePtr& eventQueue,
                             rfa::common::Client& client)
    : session_(session),
      eventQueue_(eventQueue),
      client_(client)
{
    try {
        subscriber_ = createMarketDataSubscriber(*session, RFA_String());
        handle_ = subscriber_
            ->registerClient(*eventQueue, rfa::sessionLayer
                             ::MarketDataSubscriberInterestSpec(true),
                             client, 0);
        if (!handle_)
            throw MarketDataError
                ("MarketDataSubscriber::registerClient() failed");

    } catch (const rfa::common::Exception& e) {
        throw MarketDataError(toString(e));
    }
}

 RfaSubscriptionPtr
RfaSubscriber::createSubscription(const string& serviceName,
                                  const string& itemName)
{
    RfaSubscriptionPtr subscription;

    try {
        subscription = rfaclient::createSubscription(*subscriber_, *eventQueue_,
                                                 serviceName, itemName,
                                                 client_);
    } catch (const rfa::common::Exception& e) {
        throw MarketDataError(toString(e));
    }
    return subscription;
}
