#include "cometrfapp/application.hpp"
#include "cometrfapp/manager.hpp"

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/format.hpp>

#include <iostream>

using namespace boost;
using namespace boost::posix_time;
using namespace comet;
using namespace std;

namespace {

    RfaManager* manager_ = 0;

    class TestClient : public RfaClientBase {
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       MarketDataMessageType msgType, const Fields& fields)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("service=[%1%], item=[%2%], msgtype=[%3%]")
                      % serviceName % itemName % toString(msgType)));

            Fields::const_iterator it(fields.begin()), end(fields.end());
            for (; it != end; ++it)
                writeLog(LOGINFO, SUBMARKETDATA, str
                         (format("%1%=[%2%]:%3%") % it->first % it->second
                          % toString(it->second.which())));

            if (rfa::sessionLayer::MarketDataItemEvent::Image == msgType) {

                Fields update;
                update["A_PRICE_1"] = 1.0;
                manager_->publish("FXPROP_DTS", "CCYCCY(TEST_USDMXN)",
                                  update);
            } else {

                if ((it = fields.find("A_PRICE_1")) != fields.end()) {

                    Fields update;
                    update["A_PRICE_1"] = get<Real>(it->second) + 1.0;
                    manager_->publish("FXPROP_DTS", "CCYCCY(TEST_USDMXN)",
                                      update);
                }
            }
        }
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       State state, StatusCode statusCode, const string& text)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("service=[%1%], item=[%2%], state=[%3%],"
                             " status_code=[%4%], test=[%5%]")
                      % serviceName % itemName % state % statusCode % text));
        }
        void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("contid=[%1%], state=[%2%], status_code=[%3%]")
                      % contId % state % statusCode));
        }
    };
}

int
main(int argc, char* argv[])
{
    try {
        RfaApplication rfa("RFA.cfg", "appendix_a", "EnumType.def");

        TestClient client;
        RfaManager manager("COMETSession", client, 1000);
        manager_ = &manager;
        manager.subscribe("FXPROP_DTS", "CCYCCY(TEST_USDMXN)");
        sleep(5);
        return 0;

    } catch (const rfa::common::Exception& e) {
        writeLog(LOGERROR, SUBMARKETDATA, COMET_HERE(toString(e)));
    } catch (const exception& e) {
        writeLog(LOGERROR, SUBGENERAL, COMET_HERE(e.what()));
    }
    return 1;
}
