#include "cometrfapp/application.hpp"
#include "cometrfapp/manager.hpp"

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/format.hpp>

#include <iostream>

using namespace boost;
using namespace boost::posix_time;
using namespace comet;
using namespace std;

namespace {

    class TestClient : public RfaClientBase {
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       MarketDataMessageType msgType, const Fields& fields)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("service=[%1%], item=[%2%], msgtype=[%3%]")
                      % serviceName % itemName % toString(msgType)));

            Fields::const_iterator it(fields.begin()), end(fields.end());
            for (; it != end; ++it)
                writeLog(LOGINFO, SUBMARKETDATA, str
                         (format("%1%=[%2%]:%3%")
                          % it->first
                          % it->second
                          % toString(it->second.which())));
        }
        void
        doProcessEvent(const string& serviceName, const string& itemName,
                       State state, StatusCode statusCode, const string& text)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("service=[%1%], item=[%2%], state=[%3%],"
                             " status_code=[%4%], test=[%5%]")
                      % serviceName % itemName % state % statusCode % text));
        }
        void
        doProcessEvent(long contId, ContReplyState state,
                       ContReplyStatusCode statusCode)
        {
            writeLog(LOGINFO, SUBMARKETDATA, str
                     (format("contid=[%1%], state=[%2%], status_code=[%3%]")
                      % contId % state % statusCode));
        }
    };
}

int
main(int argc, char* argv[])
{
    try {
        RfaApplication rfa("RFA.cfg", "appendix_a", "EnumType.def");

        TestClient client;
        RfaManager manager("COMETSession", client, 1000, 0);
        manager.subscribe("IDN_SELECTFEED", "EUR=");
        manager.subscribe("IDN_SELECTFEED", "CHF=");
        manager.subscribe("IDN_SELECTFEED", "JPY=");
        manager.subscribe("IDN_SELECTFEED", "GBP=");
        manager.subscribe("IDN_SELECTFEED", "AUD=");

        manager.subscribe("IDN_SELECTFEED", "CHF=EBS");
        manager.subscribe("IDN_SELECTFEED", "EUR=EBS");
        manager.subscribe("IDN_SELECTFEED", "EURCHF=EBS");
        manager.subscribe("IDN_SELECTFEED", "EURJPY=EBS");
        manager.subscribe("IDN_SELECTFEED", "GBP=D2");
        manager.subscribe("IDN_SELECTFEED", "EURGBP=D2");
        manager.subscribe("IDN_SELECTFEED", "AUD=D2");
        manager.subscribe("IDN_SELECTFEED", "CAD=D2");
        manager.subscribe("IDN_SELECTFEED", "NZD=D2");
        manager.subscribe("IDN_SELECTFEED", "EURSEK=D2");
        manager.subscribe("IDN_SELECTFEED", "EURNOK=D2");

        manager.clear();

        static const char CCYCCYS[][7] = {
            "EURUSD",
            "USDCHF",
            "EURCHF",
            "USDJPY",
            "EURJPY",
            "GBPUSD",
            "EURGBP",
            "AUDUSD",
            "USDCAD",
            "NZDUSD",
            "EURNOK",
            "EURSEK",
            "XXXYYY"
        };

        for (int i(0); i < 5; ++i) {
            for (unsigned j(0); j < sizeof(CCYCCYS) / sizeof(CCYCCYS[0]);
                 ++j) {
                writeLog(LOGINFO, SUBMARKETDATA,
                         str(format("subscribing to [%1%]") % CCYCCYS[j]));
                manager.subscribe("FXPROP_DTS", str(format("CCYCCY(%1%)")
                                                    % CCYCCYS[j]));
            }
            sleep(1);
            for (unsigned j(0); j < sizeof(CCYCCYS) / sizeof(CCYCCYS[0]);
                 ++j) {
                writeLog(LOGINFO, SUBMARKETDATA, str
                         (format("unsubscribing from [%1%]") % CCYCCYS[j]));
                manager.unsubscribe("FXPROP_DTS", str(format("CCYCCY(%1%)")
                                                      % CCYCCYS[j]));
            }
            sleep(1);
        }
        manager.clear();
        return 0;

    } catch (const rfa::common::Exception& e) {
        writeLog(LOGERROR, SUBMARKETDATA, COMET_HERE(toString(e)));
    } catch (const exception& e) {
        writeLog(LOGERROR, SUBGENERAL, COMET_HERE(e.what()));
    }
    return 1;
}
