#include "cometrfapp/application.hpp"
#include "cometrfapp/utility.hpp"

#include <boost/format.hpp>

#include <iostream>

using namespace boost;
using namespace comet;
using namespace std;

int
main(int argc, char* argv[])
{
    try {

        RfaApplication rfa("RFA.cfg", "appendix_a", "EnumType.def");

        TibField in;
        Field out;

        in.Update("ROW80_1", Tib_i32(101));
        unpack(out, in);
        writeLog(LOGINFO, SUBMARKETDATA, str(format("TIBMSG_INT [%1%]")
                                             % get<Int>(out)));
        COMET_ASSERT(get<Int>(out) == 101);

        in.Update("ROW80_1", "test");
        unpack(out, in);
        writeLog(LOGINFO, SUBMARKETDATA, str(format("TIBMSG_STRING [%1%]")
                                             % get<String>(out)));
        COMET_ASSERT(get<String>(out) == "test");

        in.Update("ROW80_1", 1000000.123456);
        unpack(out, in);
        writeLog(LOGINFO, SUBMARKETDATA, str(format("TIBMSG_REAL [%1%]")
                                             % get<Real>(out)));
        COMET_ASSERT(get<Real>(out) == 1000000.123456);

        in.Update("ROW80_1", "");
        unpack(out, in);
        writeLog(LOGINFO, SUBMARKETDATA, str(format("TIBMSG_STRING [%1%]")
                                             % get<String>(out)));
        COMET_ASSERT(get<String>(out).empty());

        in = TibField("ROW80_1", TIBMSG_STRING, 0, "");
        unpack(out, in);
        writeLog(LOGINFO, SUBMARKETDATA, str(format("TIBMSG_STRING [%1%]")
                                             % get<String>(out)));
        COMET_ASSERT(get<String>(out).empty());
        return 0;

    } catch (const rfa::common::Exception& e) {
        writeLog(LOGERROR, SUBMARKETDATA, COMET_HERE(toString(e)));
    } catch (const exception& e) {
        writeLog(LOGERROR, SUBGENERAL, COMET_HERE(e.what()));
    }
    return 1;
}
