#define rfaclientRFAPP_BUILD
#include "types.hpp"

#include "utility.hpp"

#include <boost/algorithm/string.hpp>

#include <iomanip> 

using namespace boost;
using namespace rfaclient;
using namespace std;


 const char*
rfaclient::toString(FieldType type)
{
    switch (type) {
    case NODATA:
        return "NODATA";
    case NOSUPP:
        return "NOSUPP";
    case BOOL:
        return "BOOL";
    case INT:
        return "INT";
    case UINT:
        return "UINT";
    case REAL:
        return "REAL";
    case STRING:
        return "STRING";
    case ARRAY:
        return "ARRAY";
    }
    return 0;
}

 istream&
rfaclient::readRecord(istream& is, Record& record)
{
    record.itemName_.clear();
    record.fields_.clear();

    string line;
    while (getline(is, line)) {

        // A blank line marks the record boundary.

        trim(line);
        if (line.empty())
            break;

        pair<string, string> pair(splitPair(line));
        if (pair.first == "X_RIC_NAME")
            record.itemName_ = pair.second;
        else
            record.fields_.insert(pair);
    }
    return is;
}

 ostream&
rfaclient::writeRecord(ostream& os, const string& itemName, const Fields& fields)
{
    os << "X_RIC_NAME=" << itemName << endl;

    Fields::const_iterator it(fields.begin()), end(fields.end());
    for (; it != end; ++it)
        os << it->first << '='
           << setprecision(numeric_limits<double>::digits10)
           << setiosflags(ios::fixed)
           << it->second
           << endl;

    // Blank line to terminate.

    return os << endl;
}
