#ifndef rfaclientRFAPP_TYPES_HPP
#define rfaclientRFAPP_TYPES_HPP

#include "config.hpp"

#include <boost/variant.hpp>

#include <string>
#include <map>
#include <vector>
#include <limits>

namespace rfaclient {

    struct NoData { };
    struct NoSupp { };
    typedef bool Bool;
    typedef long Int;
    typedef unsigned long UInt;
    typedef double Real;
    typedef std::string String;
    typedef std::vector<char> Array;

	/* boost::variant field equivalents */
    typedef boost::variant<
        NoData,
        NoSupp,
        Bool,
        Int,
        UInt,
        Real,
        String,
        Array
        > Field;

	/* Tib fields */
    enum FieldType {
        NODATA,
        NOSUPP,
        BOOL,
        INT,
        UINT,
        REAL,
        STRING,
        ARRAY
    };

     const char*
    toString(FieldType type);

    inline const char*
    toString(int type)
    {
        return toString(static_cast<FieldType>(type));
    }

    inline std::ostream&
    operator <<(std::ostream& os, const NoData&)
    {
        return os << "(nodata)";
    }

    inline std::ostream&
    operator <<(std::ostream& os, const NoSupp&)
    {
        return os << "(data)";
    }

    typedef std::map<std::string, Field> Fields;

    struct Record {
        std::string itemName_;
        Fields fields_;
    };

     std::istream&
    readRecord(std::istream& is, Record& record);

     std::ostream&
    writeRecord(std::ostream& os, const std::string& itemName,
                const Fields& fields);
}

namespace std {

    inline ostream&
    operator <<(ostream& os, const rfaclient::Array&)
    {
        return os << "(array)";
    }
}

#endif // rfaclientRFAPP_TYPES_HPP
